<?php #charset utf-8

/**
 Gestion des templates avec remplacement iteratif, rcursif, alternatif

 @author Bieler Batiste
 @version 0.95
 @company http://dosimple.ch

Copyright (C) 2005  Bieler Batiste

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

class TempLight{

	
function TempLight( $file, $lang=false )
{
	$this->file = $file;
	
	$this->model = $this->getTemplate($file);
	
	// start block
	$this->sB='<!--{';
	// end block
	$this->eB='}-->';
	$this->sV='<!--';
	$this->eV='-->';
	$this->separator=':';
	
	$this->compileInclude();
	$this->compileConditions();
	$this->setLang($lang);
	$this->compileLang();
}

// dfini la langue utilise dans le template
// pour grer plusieurs langues il suffit de crer un tableau
function setLang($lang)
{
	$this->lang=$lang;
}

function getTemplate($file)
{
	if( file_exists( $file ) )
	{
		return file_get_contents( $file );
	}
	else
	{
		trigger_error("TempLight : ressource $file indisponible", E_USER_ERROR);
	}
}

function compileInclude()
{
	$reg_expr='#'.$this->sB.'include'.$this->separator.'(.*?)'.$this->eB.'#si';
	while(preg_match($reg_expr,$this->model,$matches))
	{
		$this->model = preg_replace('#'.$this->sB.'include'.$this->separator.$matches[1].$this->eB.'#si',$this->getTemplate($matches[1]),$this->model);
	}
}

function compileLang()
{
	if( $this->lang )
	{
		$this->model = preg_replace('#'.$this->sB.'(end)?Lang'.$this->separator.$this->lang.$this->eB.'#si','',$this->model);
		$this->model = preg_replace('#'.$this->sB.'lang'.$this->separator.'([a-z]*)'.$this->eB.'.*?'.$this->sB.'endLang'.$this->separator.'\1'.$this->eB.'#si','',$this->model);
	}
}

// numrote les conditions afin de les identifier
// utilise une machine pile pour y arriver
function compileConditions()
{
	$i=0; // compteur de bloc de condition
	$a=array(); // la pile
	
	while(preg_match('#'.$this->sB.'(if|elseif|endif|else)([^0-9i])#si',$this->model,$matches))
	{
	if($matches[1]=='if')
	{
		$i++;
		array_push($a,$i);
	}
	
	if( count($a)==0 )
		trigger_error ( "TempLight : fermeture superflue pour la condition $i dans ".$this->file, E_USER_ERROR );
	
	$this->model = preg_replace('#'.$matches[0].'#si', $this->sB.$matches[1].$a[count($a)-1].$matches[2], $this->model, 1);

	if(strtolower($matches[1])=='endif')
	{
		array_pop($a);
	}
	}
	if( count($a)>0 )
	trigger_error ( 'TempLight : ouverture superflue dans '.$this->file, E_USER_ERROR );
}

function makeObjectFromArray(&$arrays)
{
	$root=new Node();
	$this->_makeObjectFromArrayRecursive($root,$arrays);
	return $root;
}

function _makeObjectFromArrayRecursive(&$currentNode,&$arrays)
{
	//print_r($arrays);
	$i=0;
	foreach($arrays as $key => $value)
	{
		if(is_array($value))
		{
			$i++;
			$child = new Node();
			$child->number=$i;
			$child->parentNode=$currentNode;
			$currentNode->children[]=$child;
			$this->_makeObjectFromArrayRecursive($child,$value);
		}
		else
		{
			$currentNode->fields[$key]=$value;
		}
	}
}

// remplacement simple
function replace( $tagName, $contents )
{
	$this->model = preg_replace('#'.$this->sV.$tagName.$this->eV.'#si', $contents, $this->model );
}
	
// remplacement iteratif
function iterateReplace( $iterateName, &$contentsArray )
{
	$root = $this->makeObjectFromArray($contentsArray);
	$this->_iterateReplace($iterateName, $root );
}

// remplacement iteratif, fonction interne  la classe
function _iterateReplace($iterateName, &$currentNode)
{
	$beginLoop=$this->sB.'loop'.$this->separator.$iterateName.$this->eB;
	$endLoop=$this->sB.'endLoop'.$this->separator.$iterateName.$this->eB;
	if( preg_match("#$beginLoop(.*?)$endLoop#si", $this->model, $matches ) )
	{
		$iterateContent = $matches[1];
		$result='';
		foreach($currentNode->children as $child)
		{
			$iterateContentBuffer = $this->solveAlternative($iterateContent, $child);
			foreach( $child->fields as $tag=>$value )
			{
				$iterateContentBuffer=
				preg_replace('#'.$this->sV.$tag.$this->eV.'#si', $value, $iterateContentBuffer );
			}
			$result.=$iterateContentBuffer;
		}
		$this->model = preg_replace("#$beginLoop.*?$endLoop#si", $result, $this->model );
	}
	else
	{
		trigger_error("TempLight : boucle '$iterateName' non trouve", E_USER_ERROR);
	}
}

// cherche des conditions et les rsouds
// le contexte sert  passer des variables de tests pour les alternatives
function solveAlternative($contents, &$currentNode)
{
	// on cherche des blocs conditionnels
	$reg_expr='#('.$this->sB.'if([0-9])*:.*?'.$this->eB.'.*?)'.$this->sB.'endif\2'.$this->eB.'#si';
	
	while(preg_match($reg_expr,$contents,$matches))
	{
		$nb=$matches[2];
		$reg_expr2='#('.$this->sB.'if'.$nb.'*:.*?'.$this->eB.'.*?)'.$this->sB.'endif'.$nb.$this->eB.'#sie';
		$contents = preg_replace($reg_expr2,
		"\$this->replaceAlternative(\$matches[1],\$nb,\$currentNode)", $contents );
	}
	return $contents;
}
	
function replaceAlternative($alternatives, $nb, &$currentNode)
{
	$alternativeContents = preg_split('#'.$this->sB.'(if|else|elseif)'.$nb.'(:.*?)?'.$this->eB.'#si', $alternatives, -1, PREG_SPLIT_NO_EMPTY);
	// extrait les conditions des alternative
	preg_match_all('#(if|elseif)'.$nb.':(.*?)'.$this->eB.'#si', $alternatives, $alternativeConditions, PREG_SET_ORDER);
	$i=0;
	// boucle de test
	foreach($alternativeContents as $alternativeContent)
	{
		// l'alternative existe ?
		if(isset($alternativeConditions[$i]))
		{
			$condition = $alternativeConditions[$i][2];
			$condition = preg_replace('#(\w{1,}\.)#si','$\1',$condition);
			$condition = preg_replace('#\.#si','->',$condition);
			eval('$condition='.$condition.';');
			// si elle est vraie, on renvoie le contenu associ
			if($condition)
			{
				return $alternativeContent;
			}
			// autrement on continue la recherche d'une expression vraie
		}
		// pas d'alternative, c'est donc un else
		else
		{
			return $alternativeContent;
		}
		// alternative suivante
		$i++;
	}
}
	
function recursiveReplace($tagName, $contentsArray)
{
	$root = $this->makeObjectFromArray($contentsArray);
	$reg_expr='#'.$this->sB.'recursion'.$this->separator.$tagName.$this->eB.'(.*?)'.$this->sB.
	'endRecursion'.$this->separator.$tagName.$this->eB.'#sie';
	if(preg_match($reg_expr, $this->model, $matches)==0)
		trigger_error("TempLight : recursion '$tagName' non trouve", E_USER_ERROR);
	// on extrait le contenu de la recursivit
	$recursiveContent = $matches[1];
	$this->model = preg_replace($reg_expr, "\$this->recursiveLoop(\$recursiveContent,\$root)", $this->model );
}
	
function recursiveLoop($recursiveContent, $currentNode)
{
	$loopContentArray=array();
	$reg_expr='#'.$this->sB.'loop'.$this->eB.'(.*?)'.$this->sB.'endLoop'.$this->eB.'#si';
	$i=0;
	$recursiveContentBuffer=$recursiveContent;
	while(preg_match($reg_expr, $recursiveContentBuffer, $matches))
	{
		array_push($loopContentArray,$matches[1]);
		$recursiveContentBuffer = preg_replace($reg_expr, $this->sB.'savedLoop'.$i.$this->eB, $recursiveContentBuffer, 1);
		$i++;
	}
	
	$recursiveContentBuffer = $this->solveAlternative($recursiveContentBuffer, $currentNode);
	preg_match('#'.$this->sB.'savedLoop([0-9]*)'.$this->eB.'#si',$recursiveContentBuffer, $matches);
	$iterateContent = $loopContentArray[$matches[1]];
	
	$loopResult='';
	
	foreach($currentNode->children as $child)
	{
		$iterateContentBuffer=$this->solveAlternative($iterateContent, $child);
		
		foreach($child->fields as $tag=>$value)
			{
				$iterateContentBuffer = preg_replace('#'.$this->sV.$tag.$this->eV.'#si', $value, $iterateContentBuffer);
			}
			
		$iterateContentBuffer = preg_replace('#'.$this->sB.'recursion'.$this->eB.'#si', 
			$this->recursiveLoop($recursiveContent, $child), $iterateContentBuffer);
		$iterateContentBuffer = preg_replace('#'.$this->sB.'recursion'.$this->eB.'#si', '', $iterateContentBuffer);
		
		$loopResult.=$iterateContentBuffer;
	}
	
	return preg_replace('#'.$this->sB.'savedLoop[0-9]*'.$this->eB.'#si', $loopResult, $recursiveContentBuffer );
}
	
function setContext( $contextArray )
{
	$this->contextArray=array_merge($this->contextArray,$contextArray);
}
	
// le paramtre final supprime toutes les balises de templates
function toString( $final=true )
{
	if( $final )
	{
		$sb = $this->sB;
		$eb = $this->eB;
		$buffer=$this->model;
		// rsolutions des conditions
		$buffer = $this->solveAlternative( $buffer, $this->contextArray );
		// suppression des conditions
		$buffer = preg_replace('#'.$sb.'if:.*?'.$eb.'.*?'.$sb.'endif'.$eb.'#si', '', $buffer );
		// suppression des boucles
		$buffer = preg_replace('#'.$sb.'loop:.*?'.$eb.'.*?'.$sb.'endLoop:.*?'.$eb.'#si', '', $buffer );
		// suppression des rcursions
		$buffer = preg_replace('#'.$sb.'recursion:.*?'.$eb.'.*?'.$sb.'endRecursion:.*?'.$eb.'#si', '', $buffer );
		// suppression des tables
		$buffer = preg_replace('#'.$sb.'table:.*?'.$eb.'.*?'.$sb.'endTable:.*?'.$eb.'#si', '', $buffer );
		// suppresion de tout les tags restants
		return preg_replace('#'.$sb.'.*?'.$eb.'#si', '', $buffer );
	}
	else
	{
		return $this->model;
	}
}
	
}

class Node
{
	function Node()
	{
		$this->fields=array();
		$this->number=false;
		$this->parentNode=false;
		$this->children=array();
	}
	
	function field($key)
	{
		return $this->fields[$key];
	}
	
	function level($i=1)
	{
		if($this->parentNode!=false)
		{
			$i++;
			return $this->parentNode->level($i);
		}
		return $i;
	}

	function parentNode()
	{
		return $this->parentNode;
	}
	
	function nbBrother()
	{
		return $this->parentNode->nbChildren();
	}
	
	function nbChildren()
	{
		return count($this->children);
	}
	
	function brother($index)
	{
		$i=$index+$this->number-1;
		while($i<0)
			$i+=$this->nbBrother();
		while($i>$this->nbBrother()-1)
			$i-=$this->nbBrother();
		$p = $this->parentNode();
		$c = $p->child($i);
		return $c;
	}
	
	function isFirst()
	{
		return $this->number==1;
	}
	
	function isLast()
	{
		return $this->number==$this->nbBrother();
	}
	
	function number()
	{
		return $this->number;
	}
	
	function child($index)
	{
		return $this->children[$index];
	}
}

?>
